/**
 *
 * Written by Ed Hong UWT Feb. 19, 2003
// based on code in Goodrich & Tamassia's Algorithm Design,
// page 330 & 331.
 *
 * @author edhong
 * @version 0.1
 */

import java.util.*;


/** Generic depth first search traversal of a graph using the template
 * method pattern.  A subclass should override various methods to add
 * functionality to this traversal.  */
public abstract class DFS {
    /** The graph being traversed. */
    protected SimpleGraph G = null;

    /** Perform a depth first search traversal.
     * @param g Input graph.
     * @param start Start vertex of the traversal.
     * The default version in the abstract class
     * resets the graph, and then calls dfsTraversal on v.
     * override this function if you want different behavior
     * (such as a dfs_sweep).
     */
    public void execute(SimpleGraph g,Vertex start) {
		resetGraph(g);
		dfsTraversal(start);
    }

    /**
	   Resets the edges and vertices of graph g all be unexplored
	   sets G to keep track of the graph g that we are traversing
     */
    protected void resetGraph(SimpleGraph g) {
	    Iterator i;
	    Vertex v;
	    Edge e;
	    G = g;

	    for (i= g.vertices(); i.hasNext(); ) {
	       v = (Vertex) i.next();
	       v.setLabel(UNEXPLORED);
	    }

	    for (i= g.edges(); i.hasNext();) {
	       e = (Edge) i.next();
	       e.setLabel(UNEXPLORED);
	    }
    }

    /**
     * Recursive template method for a generic DFS traversal.
     * @param v Start vertex of the traversal.
     */
    protected void dfsTraversal(Vertex v) {
	   v.setLabel(VISITED);
	   startVisit(v);
	   for (Iterator inEdges = G.incidentEdges(v); inEdges.hasNext(); ) {
	      Edge nextEdge = (Edge) inEdges.next();
	      if (nextEdge.getLabel() == UNEXPLORED) { // found an unexplored edge, explore it
		     Vertex w = G.opposite(v, nextEdge);
		     if (w.getLabel() == UNEXPLORED) { // w is unexplored, this is a discovery edge
		 	    nextEdge.setLabel(DISCOVERY);
		        traverseDiscovery(nextEdge, v);
		        if (!isDone()) {
			       dfsTraversal(w);
			    }
		     }
		     else  {// w is visited, this is a back edge
			    nextEdge.setLabel(BACK);
		        traverseBack(nextEdge, v);
	         }
		  }
	    }
	    finishVisit(v);
    }


	/** Constants for labeling vertices and edges */
	protected static Object UNEXPLORED = new Object();
	protected static Object VISITED = new Object();
	protected static Object DISCOVERY = new Object();
	protected static Object BACK = new Object();


    /** Auxiliary methods for specializing a generic DFS */
    protected void startVisit(Vertex v) {} // Called when we first visit v
    protected void finishVisit(Vertex v) {} // Called when we finish with v (all incident edges explored already)
    protected void traverseDiscovery(Edge e,Vertex from) {} // Discovery edge
    protected void traverseBack(Edge e,Vertex from) {} // Back edge
    protected boolean isDone() { return false; } // Is DFS done early?
}
